#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import ssl
import sys
import urllib2
import urllib
import json
import re
from bs4 import BeautifulSoup
from urlparse import parse_qsl

import xbmc
import xbmcaddon
import xbmcgui
import xbmcplugin

_url = sys.argv[0]
_handle = int(sys.argv[1])
_addon = xbmcaddon.Addon()
_name = _addon.getAddonInfo('name')
_icon = _addon.getAddonInfo('icon')


def URL(action, link):
    u = _url + '?action=' + urllib2.quote(action)
    if link is not None:
        u += '&link=' + urllib2.quote(link)
    return u


MENU_ITEMS = []
DOMAIN = "https://www1.putlockers.co"
UA = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.0.3) Gecko/2008092417 Firefox/3.0.3'
MAIN_MENU = [
    ("Movies", URL('list', DOMAIN + '/movies')),
    ("Top IMBd", URL('list', DOMAIN + '/topimdb')),
    ("TV Shows", URL('list', DOMAIN + '/tvseries')),
    ("Search", URL('find', None))
]


def new_menu_item(label, action, url, img=None, playable=False):
    item = xbmcgui.ListItem(label=label, thumbnailImage=img)
    if playable:
        item.setProperty('IsPlayable', 'true')
        xbmcplugin.addDirectoryItem(_handle, url, listitem=item)
    else:
        MENU_ITEMS.append((URL(action, url), item, True))
    return item


def make_soup(url, params=None, raw=False, headers=None):
    xbmc.log("Requesting URL: " + url, 4)
    if params:
        paramsdata = urllib.urlencode(params)
        req = urllib2.Request(url, paramsdata)
    else:
        req = urllib2.Request(url)
    req.add_header('User-Agent', UA)
    for k, v in (headers or {}).items():
        req.add_header(k, v)
    resp = urllib2.urlopen(req, timeout=120)
    data = resp.read()

    result = data.decode('utf-8', 'ignore')
    if not raw:
        result = BeautifulSoup(result, "html.parser")
    resp.close()
    return result


def play_video(link):
    play_item = xbmcgui.ListItem(path=link)
    # play_item.setInfo(type="video", infoLabels=static.VideoInfo.load_current().compose())
    xbmcplugin.setResolvedUrl(_handle, True, listitem=play_item)


def process_list(url):
    soup = make_soup(url)
    for video in soup.find_all('div', class_=['singleVideo']):
        title = video['title']
        url = video.find('a')['href']
        img = video.find('img')['src']
        tmp = video.find('span', class_='mli-quality')
        additional = None
        if tmp is not None:
            additional = tmp.getText()
        else:
            tmp = video.find('p', class_='seacsonInfo')
            if tmp is not None:
                additional = tmp.find('span').getText() + " Season"

        if additional:
            title = '%s [COLOR red]%s[/COLOR]' % (title, additional)
        info = video.find_all('div', class_=['jt-info'])
        year = info[1].getText().split('-')[0]
        plot = video.find('p', class_='f-desc').getText()

        info_labels = {'plot': plot}
        if str(year).isdigit():
            info_labels['year'] = year

        item = new_menu_item(title, 'video', url, img=img)
        item.setInfo(type="video", infoLabels=info_labels)
    pagination = soup.find('ul', class_='pagination')
    if pagination is None:
        return
    pagination_a = pagination.find_all('a')
    current = pagination.find_all('li', class_='active')[-1].getText()
    next_url = pagination_a[-1]['href']
    total = pagination_a[-2].getText()
    xbmc.log("Next URL: " + next_url, 4)
    new_menu_item('[COLOR blue]Page %d out of %s[/COLOR]' % (int(current) + 1, total), 'list', next_url)


def process_video(url):
    print "Processing video page"
    soup = make_soup(url)
    source = soup.find('a', class_=['thumb', 'mvi-cover'])
    if source is None:  # TV Series page
        if 'season' in url.split('/')[-1]:  # List episodes
            episodes = soup.find('div', class_='videosContainer').find_all('div', class_=['singleVideo'])
            for episode in episodes:
                title = episode.find('div', class_='tiptitle').getText()
                url = episode.find('a')['href']
                img = episode.find('img')['src']
                plot = episode.find('p', class_='f-desc').getText()
                item = new_menu_item(title, 'video', url, img=img)
                item.setInfo(type="video", infoLabels={'plot': plot})
        else:  # List seasons
            img = soup.find('div', class_='topdescriptionthumb').find('img')['src']
            plot = soup.find('div', class_='topdescriptiondesc').find('p').getText()
            for a in soup.find('div', class_='movies-letter').find_all('a'):
                item = new_menu_item('Season %s' % a.getText(), 'video', a['href'], img=img)
                item.setInfo(type="video", infoLabels={'plot': plot})
        return

    soup = make_soup(source['href'])
    source = soup.find('iframe')['src']
    js_mess = str(make_soup(source, raw=True))

    """
     Extracting mirrors
    """
    tcs = js_mess.index('var tc = ')
    tce = js_mess[tcs:].index('</script>')
    mirror_data = js_mess[tcs:tcs + tce]
    mirror_token = re.findall(r"\"_token\": \"([0-9A-Za-z].+)\"", mirror_data)[0]
    mirror_te = re.findall(r"var tc = '([0-9A-Za-z].+)'", mirror_data)[0]
    mirror_p = re.findall(r"\.slice\((\d+),(\d+)\).+\"(\d+)\"\+\"(\d+)\";", mirror_data)[0]
    mirror_xtoken = mirror_te[int(mirror_p[0]):int(mirror_p[1])][::-1]+mirror_p[2]+mirror_p[3]
    mirror_json = make_soup('https://gomostream.com/decoding_v3.php', raw=True,
                            params={
                                '_token': mirror_token,
                                'tokenCode': mirror_te
                            },
                            headers={
                                'x-token': mirror_xtoken
                            })
    xbmc.log("Token %s, TE: %s, xtoken: %s, JSON: %s" % (mirror_token, mirror_te, mirror_xtoken, mirror_json), 4)
    mirrors = json.loads(mirror_json)
    for i, mirror in enumerate(mirrors):
        if mirror.startswith('https://gomostream.com/vid/?v'):
            new_menu_item("Mirror %d" % i, 'play', mirror, playable=True)


def list_link(link=None, additional=None):
    listing = []
    if link is None:
        list_item = xbmcgui.ListItem(label="Search")
        listing.append((_url + "?action=search", list_item, True))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.endOfDirectory(_handle)


def router(paramstring):
    xbmc.log("Router: " + paramstring, 4)
    params = dict(parse_qsl(paramstring))
    if params:
        if params['action'] == 'list':
            process_list(urllib2.unquote(params['link']))

        elif params['action'] == 'video':
            process_video(urllib2.unquote(params['link']))

        elif params['action'] == 'play':
            play_video(urllib2.unquote(params['link']))

        elif params['action'] == 'find':
            text = xbmcgui.Dialog().input('Searching...')
            process_list(DOMAIN + '/topimdb?s=' + urllib.quote_plus(text.strip()))

    else:
        for name, url in MAIN_MENU:
            item = xbmcgui.ListItem(label=name)
            MENU_ITEMS.append((url, item, True))
    xbmcplugin.addDirectoryItems(_handle, MENU_ITEMS, len(MENU_ITEMS))
    xbmcplugin.endOfDirectory(_handle)


if __name__ == '__main__':
    try:
        router(sys.argv[2][1:])
    except urllib2.HTTPError as e:
        xbmcgui.Dialog().notification(_name, "Connection error {}".format(e.code))

    except urllib2.URLError as e:
        xbmcgui.Dialog().notification(_name, "Unable to connect", str(e.reason))

    except ssl.SSLError as e:
        if e.message == 'The read operation timed out':
            xbmcgui.Dialog().notification(_name, "Putlockers.co server timeout", "Try again later",
                                          icon=xbmcgui.NOTIFICATION_ERROR)
        else:
            xbmcgui.Dialog().notification(_name, "Connection error!", e.message)

    # except static.UpdateException as e:
    #     xbmcgui.Dialog().notification(_name, "Page error", e.message)
